@@ -70,6 +70,7 @@ struct _GnomePilotCappletPrivate
 	GtkWidget *devices_delete;
 	GtkWidget *devices_popup;
 
+	GtkObject *pdialog;
 	GtkWidget *pilots_menu;
 	GtkWidget *pilots_username;
 	GtkWidget *conduit_treeview;
@@ -208,7 +209,8 @@ get_widgets (GnomePilotCapplet *gpcap)
 
 #define GW(name) glade_xml_get_widget (priv->xml, name)
 
-	gtk_window_set_title (GTK_WINDOW (gpcap), _("Pilot Settings"));
+	gtk_window_set_title (GTK_WINDOW (gpcap), _("gnome-pilot Settings"));
+	gtk_dialog_set_has_separator (GTK_DIALOG (gpcap), FALSE);
 	gtk_dialog_add_button (GTK_DIALOG (gpcap),
 			       GTK_STOCK_HELP, GTK_RESPONSE_HELP);
 	gtk_dialog_add_button (GTK_DIALOG (gpcap),
@@ -379,7 +381,7 @@ gnome_pilot_capplet_create_pilots_treeview (char *name, char *string1, char *str
 						     renderer, "text", 0, NULL);
 	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview), -1, _("ID"),
 						     renderer, "text", 1, NULL);
-	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview), -1, _("UserName"),
+	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview), -1, _("Owner"),
 						     renderer, "text", 2, NULL);
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
@@ -411,10 +413,8 @@ append_devices_treeview (GnomePilotCapplet *gpcap, GPilotDevice *device, GtkTree
 	gtk_list_store_append (GTK_LIST_STORE (priv->devices_model), iter);
 	gtk_list_store_set (GTK_LIST_STORE (priv->devices_model), iter,
 			    0, device->name,
-			    1, device->port,
-			    2, device->speed,
-			    3, device_type_to_str (device->type),
-			    4, device,
+			    1, device_type_to_str (device->type),
+			    2, device,
 			    -1);
 }
 
@@ -453,18 +453,14 @@ gnome_pilot_capplet_create_devices_treeview (char *name, char *string1, char *st
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled),
 					     GTK_SHADOW_IN);
 
-	model = gtk_list_store_new (5, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING, G_TYPE_POINTER);
+	model = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);
 	treeview = gtk_tree_view_new_with_model ((GtkTreeModel *) model);
 
 	renderer = gtk_cell_renderer_text_new ();
 	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview), -1, _("Name"),
 						     renderer, "text", 0, NULL);
-	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview), -1, _("Port"),
-						     renderer, "text", 1, NULL);
-	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview), -1, _("Speed"),
-						     renderer, "text", 2, NULL);
 	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview), -1, _("Type"),
-						     renderer, "text", 3, NULL);
+						     renderer, "text", 1, NULL);
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
@@ -699,7 +695,6 @@ gpcap_pilots_add (GtkWidget *widget, gpointer user_data)
 {
 	GnomePilotCapplet *gpcap = GNOME_PILOT_CAPPLET (user_data);
 	GnomePilotCappletPrivate *priv;
-	GtkObject *dlg;
 	GPilotPilot *pilot;
 	gboolean res;
 	
@@ -707,8 +702,14 @@ gpcap_pilots_add (GtkWidget *widget, gpointer user_data)
 
 	pilot = get_default_pilot (priv->state);
 	
-	dlg = gnome_pilot_pdialog_new (priv->gpc, priv->state, pilot);
-	res = gnome_pilot_pdialog_run_and_close (GNOME_PILOT_PDIALOG (dlg), GTK_WINDOW (gpcap));
+	if (priv->pdialog == NULL)
+		priv->pdialog = gnome_pilot_pdialog_new (priv->gpc, priv->state, pilot);
+	else
+		gnome_pilot_pdialog_set_pilot(priv->pdialog, pilot);
+
+	res = gnome_pilot_pdialog_run_and_close (GNOME_PILOT_PDIALOG (priv->pdialog),
+	    GTK_WINDOW (gpcap));
+
 
 	if (!res) {
 		g_free (pilot);
@@ -732,7 +733,6 @@ gpcap_pilots_edit (GtkWidget *widget, gpointer user_data)
 {
 	GnomePilotCapplet *gpcap = GNOME_PILOT_CAPPLET (user_data);
 	GnomePilotCappletPrivate *priv;
-	GtkObject *dlg;
 	GPilotPilot *pilot;
 	GtkTreeIter iter;
 	gboolean res;
@@ -741,8 +741,13 @@ gpcap_pilots_edit (GtkWidget *widget, gpointer user_data)
 	
 	pilot = get_current_pilot (gpcap, &iter);
 	
-	dlg = gnome_pilot_pdialog_new (priv->gpc, priv->state, pilot);
-	res = gnome_pilot_pdialog_run_and_close (GNOME_PILOT_PDIALOG (dlg), GTK_WINDOW (gpcap));
+	if (priv->pdialog == NULL)
+		priv->pdialog = gnome_pilot_pdialog_new (priv->gpc, priv->state, pilot);
+	else
+		gnome_pilot_pdialog_set_pilot(priv->pdialog, pilot);
+
+	res = gnome_pilot_pdialog_run_and_close (GNOME_PILOT_PDIALOG (priv->pdialog),
+	    GTK_WINDOW (gpcap));
  
 	if (res) {
 		gtk_list_store_set (GTK_LIST_STORE (priv->pilots_model), &iter,
@@ -772,7 +777,7 @@ gpcap_pilots_delete (GtkWidget *widget, gpointer user_data)
 
 	dlg = gtk_message_dialog_new (GTK_WINDOW (gpcap), GTK_DIALOG_DESTROY_WITH_PARENT, 
 				      GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
-				      _("Are you sure you want to delete %s pilot?"), pilot->name);
+				      _("Are you sure you want to delete PDA `%s'?"), pilot->name);
 
 	if (gtk_dialog_run (GTK_DIALOG (dlg)) == GTK_RESPONSE_YES) {
 		gtk_list_store_remove (GTK_LIST_STORE (priv->pilots_model),
@@ -796,11 +801,11 @@ gpcap_pilots_popup (GtkTreeView *treeview, GdkEventButton *event, gpointer user_
 	GnomePilotCappletPrivate *priv;
 
 	GnomeUIInfo popup [] = {
-		GNOMEUIINFO_ITEM_DATA (N_("_Edit this pilot..."),
-				       N_("Edit the currently selected pilot"),
+		GNOMEUIINFO_ITEM_DATA (N_("_Edit this PDA..."),
+				       N_("Edit the currently selected PDA"),
 				       gpcap_pilots_edit, gpcap, NULL),
-		GNOMEUIINFO_ITEM_DATA (N_("_Delete this pilot"),
-				       N_("Delete the currently selected pilot"),
+		GNOMEUIINFO_ITEM_DATA (N_("_Delete this PDA"),
+				       N_("Delete the currently selected PDA"),
 				       gpcap_pilots_delete, gpcap, NULL),
 		GNOMEUIINFO_END
 	};
@@ -840,7 +845,7 @@ get_current_device (GnomePilotCapplet *gpcap, GtkTreeIter *iter)
 		return NULL;
 
 	gtk_tree_model_get (priv->devices_model, iter,
-			    4, &device,
+			    2, &device,
 			    -1);
 
 	return device;
@@ -911,9 +916,7 @@ gpcap_devices_edit (GtkWidget *widget, gpointer user_data)
 	if (res) {
 		gtk_list_store_set (GTK_LIST_STORE (priv->devices_model), &iter,
 				    0, device->name,
-				    1, device->port,
-				    2, device->speed,
-				    3, device_type_to_str (device->type),
+				    1, device_type_to_str (device->type),
 				    -1);
 		
 		gpcap_save_state (gpcap);
@@ -1277,6 +1280,7 @@ gpcap_dispose (GObject *object)
 {
 	GnomePilotCapplet *gpcap = GNOME_PILOT_CAPPLET (object);
 	GnomePilotCappletPrivate *priv;
+	GtkObjectClass *gppd_class;
 	
 	priv = gpcap->priv;
 
@@ -1297,6 +1301,13 @@ gpcap_dispose (GObject *object)
 		if (priv->conduit_popup)
 			gtk_widget_destroy (priv->conduit_popup);
 
+		/* destroy pilot dialog, if it was loaded */
+		if (priv->pdialog) {
+			gppd_class = gtk_type_class (gnome_pilot_pdialog_get_type());
+			if (GTK_OBJECT_CLASS (gppd_class)->destroy)
+				GTK_OBJECT_CLASS (gppd_class)->destroy (priv->pdialog);
+		}
+		
 		g_free (priv);
 		gpcap->priv = NULL;
 	}
