@@ -46,12 +46,19 @@ struct _GnomePilotDDialogPrivate
 
 	GtkWidget *device_name;
 	GtkWidget *device_port;
+	GtkWidget *device_port_combo;
+	GtkWidget *device_port_label;
 	GtkWidget *device_speed;
+	GtkWidget *device_speed_label;
 	GtkWidget *device_timeout;
 	GtkWidget *device_serial;
 	GtkWidget *device_usb;
 	GtkWidget *device_irda;
 	GtkWidget *device_network;
+#ifdef PILOT_LINK_0_12
+	GtkWidget *libusb_label;
+	GList *libusb_list;
+#endif
 };
 
 static void class_init (GnomePilotDDialogClass *klass);
@@ -63,6 +70,9 @@ static void init_widgets (GnomePilotDDialog *gpdd);
 static void fill_widgets (GnomePilotDDialog *gpdd);
 
 static void gpdd_destroy (GtkObject *object);
+static void set_widget_visibility_by_type(GnomePilotDDialog *gpdd, int type);
+static void network_device_toggled_callback (GtkRadioButton *network_radio,
+    void *data);
 
 GtkType
 gnome_pilot_ddialog_get_type (void)
@@ -159,13 +169,27 @@ get_widgets (GnomePilotDDialog *gpdd)
 
 	priv->device_name = GW ("device_name_entry");
 	priv->device_port = GW ("device_port_entry");
+	priv->device_port_label = GW ("device_port_label");
+	priv->device_port_combo = GW ("device_port_combo");
 	priv->device_speed = GW ("device_speed_menu");
+	priv->device_speed_label = GW ("device_speed_label");
 	priv->device_timeout = GW ("timeout_spinner");
 	priv->device_serial = GW ("serial_radio");
 	priv->device_usb = GW ("usb_radio");
 	priv->device_irda = GW ("irda_radio");
 	priv->device_network = GW ("network_radio");
 
+#ifdef PILOT_LINK_0_12
+	/* usb: (libusb) pseudo-device is available from pilot-link 0.12.0 */
+	priv->libusb_list = NULL;
+	priv->libusb_label = gtk_list_item_new_with_label ("usb:");
+	gtk_widget_show(priv->libusb_label);
+	priv->libusb_list = g_list_append(priv->libusb_list,
+	    priv->libusb_label);
+	gtk_list_insert_items (GTK_LIST((GTK_COMBO(priv->device_port_combo))->list),
+	    priv->libusb_list, 1);
+#endif
+	
 #undef GW
 	return (priv->dialog
 		&& priv->device_name
@@ -204,6 +228,8 @@ init_widgets (GnomePilotDDialog *gpdd)
 
 	gtk_signal_connect (GTK_OBJECT (priv->device_port),"insert-text",
 			    GTK_SIGNAL_FUNC (insert_device_callback), NULL);
+	gtk_signal_connect (GTK_OBJECT (priv->device_network), "toggled",
+			    GTK_SIGNAL_FUNC (network_device_toggled_callback), gpdd);
 }
 
 static void
@@ -229,6 +255,7 @@ fill_widgets (GnomePilotDDialog *gpdd)
 					      priv->device->type == PILOT_DEVICE_IRDA);
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->device_network),
 					      priv->device->type == PILOT_DEVICE_NETWORK);
+		set_widget_visibility_by_type(gpdd, priv->device->type);
 	}
 }
 
@@ -237,21 +264,28 @@ gnome_pilot_ddialog_run_and_close (GnomePilotDDialog *gpdd, GtkWindow *parent)
 {
 	GnomePilotDDialogPrivate *priv;
 	gint btn;
+	GPilotDevice *tmpdev = g_new0 (GPilotDevice, 1);
 	
 	priv = gpdd->priv;
-	
-	gnome_dialog_set_parent (GNOME_DIALOG (priv->dialog), parent);
-	btn = gnome_dialog_run (GNOME_DIALOG (priv->dialog));
-	
-	if (btn == 0)
-		read_device_config (GTK_OBJECT (gpdd), priv->device);
-	
-	gnome_dialog_close (GNOME_DIALOG (priv->dialog));
 
-	if (btn == 0)
-		check_device_settings (priv->device);
+	gtk_window_set_transient_for (GTK_WINDOW (priv->dialog), parent);
+	while(1) {
+		btn = gtk_dialog_run (GTK_DIALOG (priv->dialog));
+	
+		if (btn == GTK_RESPONSE_OK) {
+			read_device_config (GTK_OBJECT (gpdd), tmpdev);
+			if(check_device_settings (tmpdev)) {
+				*priv->device = *tmpdev;
+				break;
+			}
+		} else {
+			break;
+		}
+	}
+	g_free(tmpdev);
+	gtk_widget_hide (priv->dialog);
 
-	return btn == 0 ? TRUE : FALSE;
+	return btn == GTK_RESPONSE_OK ? TRUE : FALSE;
 }
 
 static void
@@ -268,3 +302,36 @@ gpdd_destroy (GtkObject *object)
 	if (GTK_OBJECT_CLASS (parent_class)->destroy)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
+
+static void
+network_device_toggled_callback (GtkRadioButton *network_radio, void *data)
+{
+	GnomePilotDDialog *gpdd = (GnomePilotDDialog *)data;
+	GnomePilotDDialogPrivate *priv;
+	int type;
+
+	priv = gpdd->priv;
+
+	if(GTK_TOGGLE_BUTTON(network_radio)->active) {
+		type = PILOT_DEVICE_NETWORK;
+	} else {
+		type = PILOT_DEVICE_SERIAL;
+	}
+	set_widget_visibility_by_type(gpdd, type);
+}	
+
+static void
+set_widget_visibility_by_type(GnomePilotDDialog *gpdd, int type) {
+	GnomePilotDDialogPrivate *priv;
+
+	priv = gpdd->priv;
+
+	gtk_widget_set_sensitive(priv->device_port_combo,
+	    type != PILOT_DEVICE_NETWORK);
+	gtk_widget_set_sensitive(priv->device_port_label,
+	    type != PILOT_DEVICE_NETWORK);
+	gtk_widget_set_sensitive(priv->device_speed,
+	    type != PILOT_DEVICE_NETWORK);
+	gtk_widget_set_sensitive(priv->device_speed_label,
+	    type != PILOT_DEVICE_NETWORK);
+}
