@@ -21,7 +21,11 @@
  *          Robert Mibus
  */
 
-/* $Id: backup_conduit.c,v 1.42 2001/10/17 07:53:48 eskil Exp $ */
+/* $Id: backup_conduit.c,v 1.55 2006/06/18 22:24:55 mcdavey Exp $ */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <glib.h>
 #include <gnome.h>
@@ -36,6 +40,7 @@
 
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <ctype.h>
 #include <utime.h>
 #include <unistd.h>
 #include <pwd.h>
@@ -58,6 +63,7 @@ typedef struct file_db {
 	gint entries;
 } file_db;
 
+#ifdef DEBUG_REC_IO
 static char*
 pi_unmktag (long type) {
 	static char tag[5];
@@ -68,9 +74,13 @@ pi_unmktag (long type) {
 	tag[4] = 0;
 	return tag;
 }
+#endif
 
 GnomePilotConduit *conduit_get_gpilot_conduit (guint32 pilotId);
 void conduit_destroy_gpilot_conduit (GnomePilotConduit *conduit);
+void error_dialog (GtkWindow *parent, gchar *mesg, ...);
+gboolean check_base_directory (const gchar *dir_name);
+GnomePilotConduit *conduit_load_gpilot_conduit (GPilotPilot *pilot);
 
 static void 
 load_configuration(GnomePilotConduit *conduit, 
@@ -113,7 +123,8 @@ load_configuration(GnomePilotConduit *conduit,
 	if ((*c)->backup_dir == NULL) {
 		if (conduit != NULL && GNOME_IS_PILOT_CONDUIT (conduit)) {
 			(*c)->backup_dir = g_strdup (gnome_pilot_conduit_get_base_dir (conduit));
-		} else {
+		} 
+		if ((*c)->backup_dir == NULL) {
 			(*c)->backup_dir = g_strdup (g_get_home_dir ());
 		}
 	}
@@ -138,18 +149,20 @@ load_configuration(GnomePilotConduit *conduit,
 	if((*c)->backup_dir != NULL) {
 		(*c)->files_in_backup = NULL;
 		dir = opendir ((*c)->backup_dir);
-		while ((entry = readdir (dir))) {
-			if (entry->d_name) {
-				if (strlen(entry->d_name) > 4) { /* ignores ., .., 0-9999 */
-					temp_name = g_strdup_printf ("%s/%s", 
-								     ((*c)->backup_dir), 
-								     entry->d_name);
-					(*c)->files_in_backup = g_list_prepend ((*c)->files_in_backup, 
-										temp_name);
+		if (dir) {
+			while ((entry = readdir (dir))) {
+				if (entry->d_name) {
+					if (strlen(entry->d_name) > 4) { /* ignores ., .., 0-9999 */
+						temp_name = g_strdup_printf ("%s/%s", 
+									     ((*c)->backup_dir), 
+									     entry->d_name);
+						(*c)->files_in_backup = g_list_prepend ((*c)->files_in_backup, 
+											temp_name);
+					}
 				}
 			}
+			closedir (dir);
 		}
-		closedir (dir);
 	}
 
 	(*c)->pilotId = pilot->pilot_id;
@@ -161,10 +174,9 @@ static void
 save_configuration(ConduitCfg *c) 
 {
 	gchar *prefix;
-	gchar **exclude;
-	gchar *exc=NULL;
+	const gchar **exclude = NULL;
 	GList *iterator;
-	guint i;
+	guint i = 0;
 
 	g_return_if_fail(c!=NULL);
        	
@@ -177,20 +189,18 @@ save_configuration(ConduitCfg *c)
 	    exclude[i] = iterator->data;
 	  }
 	  exclude[i] = NULL;
-	  exc = g_strjoinv( ",", exclude);
-	  g_free(exclude);
 	}
 	gnome_config_push_prefix(prefix);
 	gnome_config_set_string("backup_dir",c->backup_dir);
 	gnome_config_set_bool("updated_only",c->updated_only);
 	gnome_config_set_bool("remove_deleted",c->remove_deleted);
 	gnome_config_set_int("no_of_backups",c->no_of_backups);
-	gnome_config_set_string("exclude_files",exc);
+	gnome_config_set_vector("exclude_files", i, exclude);
 	gnome_config_pop_prefix();
 	gnome_config_sync();
 	gnome_config_drop_all();
 	g_free(prefix);
-	g_free(exc);
+	g_free(exclude);
 }
 
 static void 
@@ -260,6 +270,7 @@ destroy_configuration(ConduitCfg **c)
 	g_list_foreach ((*c)->exclude_files,(GFunc)g_free,NULL);
 	g_list_free ((*c)->exclude_files);
 	g_free ((*c)->backup_dir);
+	g_free ((*c)->old_backup_dir);
 	g_free (*c);
 	*c = NULL;
 }
@@ -374,7 +385,7 @@ gnome_real_pilot_conduit_backup_backup (GnomePilotConduitBackup *conduit,
 					gpointer _cfg)
 {
 	char *name;
-	struct pi_file *f;
+	struct pi_file *f = NULL;
 	struct stat statb;
 	struct utimbuf times;
 	GList *iterator;
@@ -384,9 +395,14 @@ gnome_real_pilot_conduit_backup_backup (GnomePilotConduitBackup *conduit,
 	int keep_reading;
 	int entries;
 	int err;
-	unsigned char *buffer;
 	PilotRecord remote;
 	int wrote;
+#ifdef PILOT_LINK_0_12
+	pi_buffer_t *piBuf = NULL;
+#else
+	unsigned char *buffer = NULL;
+#endif
+	int len;
 
 	g_return_val_if_fail (conduit != NULL, -1);
 	g_return_val_if_fail (dbinfo != NULL, -1);
@@ -401,14 +417,13 @@ gnome_real_pilot_conduit_backup_backup (GnomePilotConduitBackup *conduit,
 		}
 	}
 
-	if (cfg->backup_dir == NULL) {
-		g_warning("backup conduit has no backupdir");
+	if (!g_file_test (cfg->backup_dir, G_FILE_TEST_IS_DIR | G_FILE_TEST_EXISTS)) {
+		g_warning("backup conduit has no usable backupdir");
 		gnome_pilot_conduit_send_error(GNOME_PILOT_CONDUIT(conduit),
-					       "No backup directory specified");
+					       "No usable backup directory specified");
 		result = -1;
 		goto exit;
-	} 
-
+	}
 
 	name = gnome_pilot_conduit_backup_create_name (conduit, dbinfo, cfg);
 	
@@ -441,7 +456,7 @@ gnome_real_pilot_conduit_backup_backup (GnomePilotConduitBackup *conduit,
 
 	/* backup new db */
 	f = pi_file_create (name, PI_DBINFO (dbinfo));
-	if(f==0) {
+	if(f == NULL) {
 		g_warning(_("Could not create backup file %s"),name);
 		gnome_pilot_conduit_send_error (GNOME_PILOT_CONDUIT(conduit),
 						_("Could not create backup file %s"),name);
@@ -469,23 +484,33 @@ gnome_real_pilot_conduit_backup_backup (GnomePilotConduitBackup *conduit,
 		g_warning ("error (%s) in reading '%s'", dlp_strerror (err), PI_DBINFO (dbinfo)->name);
 		result = -1;
 		goto db_close;
-	}		
-
-	buffer = g_new0 (unsigned char, 0xffff);
-	remote.buffer = buffer;
-
-	{
-		int l;
-		l = dlp_ReadAppBlock (dbinfo->pilot_socket,
-				      dbinfo->db_handle,
-				      0,
-				      buffer,
-				      0xffff);
-		if (l>0) {
-			pi_file_set_app_info (f, buffer, l);
-		}
 	}
-	
+
+
+#ifdef PILOT_LINK_0_12
+       piBuf = pi_buffer_new (0xffff);
+       len = dlp_ReadAppBlock (dbinfo->pilot_socket,
+                               dbinfo->db_handle,
+                               0,
+                               -1,
+                               piBuf);
+#else
+       buffer = g_new0 (unsigned char, 0xffff);
+       remote.buffer = buffer;
+
+       len = dlp_ReadAppBlock (dbinfo->pilot_socket,
+			       dbinfo->db_handle,
+			       0,
+			       buffer,
+			       0xffff);
+#endif
+
+       if (len > 0)
+#ifdef PILOT_LINK_0_12
+               pi_file_set_app_info (f, piBuf->data, len);
+#else
+	       pi_file_set_app_info (f, buffer, len);
+#endif
 
 	index = 0;
 	keep_reading = 1;
@@ -495,65 +520,121 @@ gnome_real_pilot_conduit_backup_backup (GnomePilotConduitBackup *conduit,
 		int id;
 			
 		if (PI_DBINFO (dbinfo)->flags & dlpDBFlagResource) {
+#ifdef PILOT_LINK_0_12
 			keep_reading = (dlp_ReadResourceByIndex (dbinfo->pilot_socket, 
 								 dbinfo->db_handle,
 								 index, 
-								 remote.buffer,
+								 piBuf,
 								 &type,
+								 &id) >= 0);
+#else
+                        keep_reading = (dlp_ReadResourceByIndex (dbinfo->pilot_socket,
+                                                                 dbinfo->db_handle,
+                                                                 index,
+								 remote.buffer,
+                                                                 &type,
 								 &id,
 								 &remote.len) >= 0);
+			
+#endif
+
 #ifdef DEBUG_REC_IO
 			g_message ("read resource %d, type = %s, size %d, , index %d/%d",
+#ifdef PILOT_LINK_0_12
+				   id, pi_unmktag (type), piBuf->used,
+#else
 				   id, pi_unmktag (type), remote.len,
+#endif
 				   index, entries);
 #endif
-			if (keep_reading) {				
-				err = pi_file_append_resource (f, 
-							       remote.buffer, 
-							       remote.len, 
+			if (keep_reading > 0) {
+#ifdef PILOT_LINK_0_12
+				err = pi_file_append_resource (f,
+							       piBuf->data,
+							       piBuf->used,
 							       type,
 							       id);
+#else
+                               err = pi_file_append_resource (f,
+                                                              remote.buffer,
+                                                              remote.len,
+							      type,
+							      id);
+#endif
 				if (err < 0) {
 					g_warning ("error in writing to file");
 				} else {
 					wrote++;
 #ifdef DEBUG_REC_IO
 					g_message ("write resource %d, type = %s, size %d, index %d/%d",
+#ifdef PILOT_LINK_0_12
+						   id, pi_unmktag (type), piBuf->used,
+#else
 						   id, pi_unmktag (type), remote.len,
+#endif
 						   index, entries);
 #endif
 				}
 			}
 		} else {
+#ifdef PILOT_LINK_0_12
 			keep_reading = (dlp_ReadRecordByIndex (dbinfo->pilot_socket, 
 							       dbinfo->db_handle,
 							       index, 
-							       remote.buffer,
+							       piBuf,
 							       &remote.recID,
-							       &remote.len,
 							       &remote.flags,
 							       &remote.catID) >= 0);
+#else
+                        keep_reading = (dlp_ReadRecordByIndex (dbinfo->pilot_socket,
+                                                               dbinfo->db_handle,
+                                                               index,
+							       remote.buffer,
+                                                               &remote.recID,
+							       &remote.len,
+                                                               &remote.flags,
+                                                               &remote.catID) >= 0);
+
+#endif
 
 #ifdef DEBUG_REC_IO
 			g_message ("read record %d, size %d, index %d/%d",
+#ifdef PILOT_LINK_0_12
+				   remote.recID, piBuf->used,
+#else
 				   remote.recID, remote.len,
+#endif
 				   index, entries);
 #endif
 
-			if (keep_reading) {
+			if (keep_reading > 0) {
+#ifdef PILOT_LINK_0_12
 				err = pi_file_append_record (f, 
-							     remote.buffer, 
-							     remote.len, 
+							     piBuf->data, 
+							     piBuf->used, 
 							     remote.flags,
 							     remote.catID,
 							     remote.recID);
+#else
+                                err = pi_file_append_record (f,
+							     remote.buffer,
+							     remote.len,
+                                                             remote.flags,
+                                                             remote.catID,
+                                                             remote.recID);
+
+#endif
 				if (err < 0) {
 					g_warning ("error in writing to file");
 				} else {
 					wrote++;
 #ifdef DEBUG_REC_IO
 					g_message ("write record %d, size %d, index %d/%d",
+#ifdef PILOT_LINK_0_12
+						   id, piBuf->used,
+#else
 						   id, remote.len,
+#endif
 						   index, entries);
 #endif
 				}
@@ -565,24 +646,38 @@ gnome_real_pilot_conduit_backup_backup (GnomePilotConduitBackup *conduit,
 		gnome_pilot_conduit_send_progress (GNOME_PILOT_CONDUIT (conduit),
 						   entries,
 						   index);
-	} while (keep_reading && index < entries);				  
+	} while ((keep_reading > 0) && (index < entries));
 
+	if (pi_file_close (f) < 0) {
+		g_warning("backup conduit can't write file");
+		gnome_pilot_conduit_send_error(GNOME_PILOT_CONDUIT(conduit),
+					       "Unable to write file to backup directory specified");
+		result = -1;
+		f = NULL;
+		goto db_close;
+	}
+	f = NULL;
+	
 	g_message ("Wrote %d of %d %s, which is %s",
 		   wrote, entries, PI_DBINFO (dbinfo)->flags & dlpDBFlagResource ? "resources" : "records",
 		   wrote == entries ? "good" : "BAD");
 
-	g_free (buffer);
-		
-	//dlp_ResetSyncFlags (dbinfo->pilot_socket, dbinfo->db_handle); 		
-
-	pi_file_close (f);
 	times.actime = PI_DBINFO (dbinfo)->createDate;
 	times.modtime = PI_DBINFO (dbinfo)->modifyDate;
 	utime (name, &times);
  db_close:
 	dlp_CloseDB (dbinfo->pilot_socket, dbinfo->db_handle);
  exit_and_free:
+	if(f != NULL)
+		pi_file_close(f);
 	g_free (name);
+#ifdef PILOT_LINK_0_12
+	if (piBuf) {
+		pi_buffer_free (piBuf);
+	}
+#else
+	g_free (buffer);
+#endif
  exit:	
 	return result;	
 }
@@ -614,14 +709,13 @@ gnome_real_pilot_conduit_backup_restore (GnomePilotConduitBackup *conduit,
 {
 	DIR *dir;
 	struct dirent *dirent;
-	GnomePilotDBInfo info;
 	file_db **db;
 	int dbcount = 0;
-	int i, j, max, size;
+	int i, j, size;
 	struct pi_file *file;
 	char *source;
 	int result = 0;
-	int reset = 0;
+
 	ConduitCfg *cfg = (ConduitCfg*)_cfg;
 
 	if (src_dir == NULL) {
@@ -672,7 +766,7 @@ gnome_real_pilot_conduit_backup_restore (GnomePilotConduitBackup *conduit,
 
 		/* Find biggest record */
 		for (i = 0; i < db[dbcount]->entries; i++) {
-			if (PI_DBINFO (&info)->flags & dlpDBFlagResource)
+			if (db[dbcount]->info.flags & dlpDBFlagResource)
 				pi_file_read_resource(file, i, 0, &size, 0, 0);
 			else
 				pi_file_read_record(file, i, 0, &size, 0, 0,0 );
@@ -704,7 +798,6 @@ gnome_real_pilot_conduit_backup_restore (GnomePilotConduitBackup *conduit,
 		}
         }
 
- exit_and_free:
 #ifdef DEBUG
 	fprintf (stderr,"backup_conduit: Restore done\n");
 #endif
@@ -714,7 +807,7 @@ gnome_real_pilot_conduit_backup_restore (GnomePilotConduitBackup *conduit,
 	}
 
 	g_free(db);
- exit:
+
 	return result;
 }
 
@@ -726,7 +819,7 @@ insert_dir_callback (GtkEditable *editable, const gchar *text,
 		     gint len, gint *position, void *data)
 {
 	gint i;
-	gchar *curname;
+	const gchar *curname;
 
 	curname = gtk_entry_get_text(GTK_ENTRY(editable));
 	if (*curname == '\0' && len > 0) {
@@ -760,7 +853,8 @@ static GtkWidget
 	gtk_table_set_col_spacings(GTK_TABLE(table), 10);
 	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, GNOME_PAD);
 
-	label = gtk_label_new(_("Backup directory"));
+	label = gtk_label_new(_("Backup directory:"));
+	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
     
 	entry = gtk_entry_new_with_max_length(128);
@@ -770,21 +864,24 @@ static GtkWidget
 			   GTK_SIGNAL_FUNC(insert_dir_callback),
 			   NULL);
 
-	label = gtk_label_new(_("Only backup changed bases"));
+	label = gtk_label_new(_("Only backup changed bases:"));
+	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1,2);
 
 	button = gtk_check_button_new();
 	gtk_object_set_data(GTK_OBJECT(vbox), "only_changed", button);
 	gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 1,2);
 
-	label = gtk_label_new(_("Remove local base if deleted on pilot"));
+	label = gtk_label_new(_("Remove local base if deleted on PDA:"));
+	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2,3);
 
 	button = gtk_check_button_new();
 	gtk_object_set_data(GTK_OBJECT(vbox), "remove_local", button);
 	gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 2,3);
 
-	label = gtk_label_new(_("# of old backups to keep"));
+	label = gtk_label_new(_("# of old backups to keep:"));
+	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 3,4);
 
 	adjustment = gtk_adjustment_new (0, 0, 100, 1, 1, 1);
@@ -795,6 +892,69 @@ static GtkWidget
 	return vbox;
 }
 
+void
+error_dialog (GtkWindow *parent, gchar *mesg, ...)
+{
+        GtkWidget *dlg;
+        char *tmp;
+        va_list ap;
+
+        va_start (ap,mesg);
+        tmp = g_strdup_vprintf (mesg,ap);
+
+        dlg = gtk_message_dialog_new (parent, GTK_DIALOG_DESTROY_WITH_PARENT,
+                                      GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, tmp);
+        gtk_dialog_run (GTK_DIALOG (dlg));
+        gtk_widget_destroy (dlg);
+
+        va_end (ap);
+        g_free (tmp);
+}
+
+gboolean
+check_base_directory (const gchar *dir_name)
+{
+        gboolean ret = TRUE;
+        /* check basedir validity */
+
+        if (mkdir (dir_name, 0700) < 0 ) {
+                struct stat buf;
+                gchar *errstr;
+                switch (errno) {
+                case EEXIST:
+                        stat (dir_name, &buf);
+                        if (S_ISDIR (buf.st_mode)) {
+                                if (!(buf.st_mode & (S_IRUSR | S_IWUSR |S_IXUSR))) {
+                                        error_dialog (NULL, _("The specified backup directory exists but has the wrong permissions.\n"
+                                                        "Please fix or choose another directory"));
+                                        ret = FALSE;
+                                }
+                        } else {
+                                error_dialog (NULL, _("The specified backup directory exists but is not a directory.\n"
+                                                "Please make it a directory or choose another directory"));
+                                ret = FALSE;
+                        }
+                        break;
+
+                case EACCES:
+                        error_dialog (NULL, _("It wasn't possible to create the specified backup directory.\n"
+                                        "Please verify the permissions on the specified path or choose another directory"));
+                        ret = FALSE;
+			break;
+                case ENOENT:
+                        error_dialog (NULL, _("The path specified for the backup directory is invalid.\n"
+                                        "Please choose another directory"));
+                        ret = FALSE;
+                        break;
+                default:
+                        errstr = strerror (errno);
+                        error_dialog (NULL, errstr);
+                        ret = FALSE;
+                }
+        }
+        return ret;
+}
+
 static void
 setOptionsCfg(GtkWidget *pilotcfg, ConduitCfg *state)
 {
@@ -811,7 +971,8 @@ setOptionsCfg(GtkWidget *pilotcfg, ConduitCfg *state)
 	g_assert(remove_deleted!=NULL);
 	g_assert(adj!=NULL);
 
-	gtk_entry_set_text(GTK_ENTRY(dir), state->backup_dir);
+	state->old_backup_dir = g_strdup(state->backup_dir);
+	gtk_entry_set_text(GTK_ENTRY(dir), state->old_backup_dir);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(updated_only), state->updated_only);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(remove_deleted), state->remove_deleted);
 	gtk_adjustment_set_value (GTK_ADJUSTMENT(adj), state->no_of_backups);
@@ -829,7 +990,14 @@ readOptionsCfg(GtkWidget *pilotcfg, ConduitCfg *state)
 	remove_deleted = gtk_object_get_data(GTK_OBJECT(pilotcfg), "remove_local");
 	adj = gtk_object_get_data(GTK_OBJECT(pilotcfg), "no_of_backups");
 
-	state->backup_dir = g_strdup(gtk_entry_get_text(GTK_ENTRY(dir)));
+	if(state->backup_dir)
+		g_free(state->backup_dir);	
+	if(check_base_directory ( gtk_entry_get_text (GTK_ENTRY (dir)))){
+		state->backup_dir = g_strdup(gtk_entry_get_text(GTK_ENTRY(dir)));
+	} else {
+		state->backup_dir = g_strdup(state->old_backup_dir);
+		gtk_entry_set_text(GTK_ENTRY(dir), state->old_backup_dir);
+	}
 	state->updated_only = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(updated_only));
 	state->remove_deleted = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(remove_deleted));
 	state->no_of_backups = GTK_ADJUSTMENT(adj)->value;
@@ -879,9 +1047,9 @@ GnomePilotConduit *
 conduit_load_gpilot_conduit (GPilotPilot *pilot)
 {
 	ConduitCfg *cfg, *cfg2;
-	GtkObject *retval;
+	GnomePilotConduitBackup *retval;
 
-	retval = gnome_pilot_conduit_backup_new (pilot);
+	retval = GNOME_PILOT_CONDUIT_BACKUP(gnome_pilot_conduit_backup_new (pilot));
 	g_assert (retval != NULL);
 
 	load_configuration(GNOME_PILOT_CONDUIT (retval), &cfg, pilot);
@@ -891,28 +1059,23 @@ conduit_load_gpilot_conduit (GPilotPilot *pilot)
 	gtk_object_set_data(GTK_OBJECT(retval),OBJ_DATA_CONFIG,cfg);
 	gtk_object_set_data(GTK_OBJECT(retval),OBJ_DATA_OLDCONFIG,cfg2);
 
-	gtk_object_set_data(retval,
+	gtk_object_set_data(GTK_OBJECT(retval),
 			    "configuration",
 			    cfg);
 
-	gtk_signal_connect (retval, 
-			    GNOME_PILOT_CONDUIT_BACKUP_SIGNAL_BACKUP (gnome_real_pilot_conduit_backup_backup), 
-			    cfg);
-	gtk_signal_connect (retval, 
-			    GNOME_PILOT_CONDUIT_BACKUP_SIGNAL_RESTORE (gnome_real_pilot_conduit_backup_restore),
-			    cfg);
-	gtk_signal_connect (retval, 
-			    GNOME_PILOT_CONDUIT_SIGNAL_CREATE_SETTINGS_WINDOW (create_settings_window), 
-			    NULL);
-	gtk_signal_connect (retval, 
-			    GNOME_PILOT_CONDUIT_SIGNAL_DISPLAY_SETTINGS (display_settings), 
-			    NULL);
-	gtk_signal_connect (retval, 
-			    GNOME_PILOT_CONDUIT_SIGNAL_SAVE_SETTINGS (save_settings), 
-			    NULL);
-	gtk_signal_connect (retval, 
-			    GNOME_PILOT_CONDUIT_SIGNAL_REVERT_SETTINGS (revert_settings), 
-			    NULL);
+	gnome_pilot_conduit_backup_connect__backup (retval, gnome_real_pilot_conduit_backup_backup, 
+					    cfg);
+	gnome_pilot_conduit_backup_connect__restore (retval, gnome_real_pilot_conduit_backup_restore, 
+					     cfg);
+	gnome_pilot_conduit_connect__create_settings_window (GNOME_PILOT_CONDUIT(retval),
+	    create_settings_window, NULL);
+	gnome_pilot_conduit_connect__display_settings (GNOME_PILOT_CONDUIT(retval),
+	    display_settings, NULL);
+	gnome_pilot_conduit_connect__save_settings (GNOME_PILOT_CONDUIT(retval),
+	    save_settings, NULL);
+	gnome_pilot_conduit_connect__revert_settings (GNOME_PILOT_CONDUIT(retval),
+	    revert_settings, NULL);	
+
 	return GNOME_PILOT_CONDUIT (retval);
 }
 
