@@ -20,7 +20,7 @@
  *          Vadim Strizhevsky
  */
 
-/* $Id: file_conduit.c,v 1.20 2001/09/17 04:34:18 eskil Exp $ */
+/* $Id: file_conduit.c,v 1.24 2006/04/20 21:12:25 mcdavey Exp $ */
 
 #include <glib.h>
 #include <gnome.h>
@@ -62,6 +62,7 @@
 /* pilot stuff end */
 
 GnomePilotConduit *conduit_get_gpilot_conduit (guint32 pilotId);
+GnomePilotConduit *conduit_load_gpilot_conduit (GPilotPilot *pilot);
 void conduit_destroy_gpilot_conduit (GnomePilotConduit *conduit);
 
 typedef struct file_db {
@@ -73,6 +74,19 @@ typedef struct file_db {
         gint maxblock;
 } file_db;
 
+#ifdef DEBUG_REC_IO
+static char*
+pi_unmktag (long type) {
+	static char tag[5];
+	tag[0] = (type >> 24) & 0xff;
+	tag[1] = (type >> 16) & 0xff;
+	tag[2] = (type >> 8) & 0xff;
+	tag[3] = type & 0xff;
+	tag[4] = 0;
+	return tag;
+}
+#endif
+
 /*
   Warning, Danger, Alert, Hot Mocha!
 
@@ -92,18 +106,19 @@ gnome_real_pilot_conduit_file_install_db (GnomePilotConduitFile *conduit,
 {
 	int index = 0;
 	int keep_reading = 1;
-	PilotRecord remote;	       	      
-	unsigned char *buffer;
-	unsigned char *appblock;
-	int appblocklen;
+	int wrote;
+	void *buffer;
+	int buffer_len;
 	int err;
 	int handle;
-	recordid_t newid;
+	PilotRecord remote;
 	int entries;
 	struct DBInfo dbi;	
 	struct pi_file *file;
 	int result = 0;
 	int reset = 0;
+	int version;
+	gboolean free_appblock = FALSE;
 
 #ifdef DEBUG
 	g_print ("in gnome_real_pilot_conduit_file_install_db\nsrc_file is %s\n", src_file);
@@ -118,6 +133,8 @@ gnome_real_pilot_conduit_file_install_db (GnomePilotConduitFile *conduit,
 		goto exit;
 	}
 
+	version = pi_version (psock);
+
 	/* Open input file */
 	file = pi_file_open(src_file);
 	if (file == NULL) {
@@ -219,7 +236,7 @@ gnome_real_pilot_conduit_file_install_db (GnomePilotConduitFile *conduit,
 					  cardno, dbi.flags, dbi.version,
 					  dbi.name, &handle) < 0) {
 				g_warning ("error (%d) in creating '%s'", err, dbi.name);
-				g_warning ("creator = %d\ntype = %d\nflags = %d\nversion = %d\n",
+				g_warning ("creator = %lu\ntype = %lu\nflags = %u\nversion = %u\n",
 					   dbi.creator, dbi.type, 
 					   dbi.flags, dbi.version);
 				gnome_pilot_conduit_send_error(GNOME_PILOT_CONDUIT(conduit),
@@ -229,7 +246,7 @@ gnome_real_pilot_conduit_file_install_db (GnomePilotConduitFile *conduit,
                         }
                 } else {	
 			g_warning ("error (%d) in creating '%s'", err, dbi.name);
-			g_warning ("creator = %d\ntype = %d\nflags = %d\nversion = %d\n",
+			g_warning ("creator = %lu\ntype = %lu\nflags = %u\nversion = %u\n",
 				   dbi.creator, dbi.type, 
 				   dbi.flags, dbi.version);
 			gnome_pilot_conduit_send_error(GNOME_PILOT_CONDUIT(conduit),
@@ -240,21 +257,44 @@ gnome_real_pilot_conduit_file_install_db (GnomePilotConduitFile *conduit,
 	}
 
 	/* Get and load the AppInfo block (if any) */
-	pi_file_get_app_info (file, (void**)&appblock, &appblocklen);	
-	if (appblocklen > 0) {
-		dlp_WriteAppBlock (psock, handle, appblock, appblocklen);
+	pi_file_get_app_info (file, &buffer, &buffer_len);	
+
+        /* Compensate for bug in OS 2.x Memo */
+        if ((version > 0x0100) && (strcmp(dbi.name, "MemoDB") == 0)
+            && (buffer_len > 0) && (buffer_len < 282)) {
+
+                /* Justification: The appInfo structure was accidentally
+                   lengthend in OS 2.0, but the Memo application does not
+                   check that it is long enough, hence the shorter block
+                   from OS 1.x will cause the 2.0 Memo application to lock
+                   up if the sort preferences are modified. This code
+                   detects the installation of a short app info block on a
+                   2.0 machine, and lengthens it. This transformation will
+                   never lose information. */
+
+                void *buffer_two = calloc(1, 282);
+
+                memcpy(buffer_two, buffer, buffer_len);
+                buffer = buffer_two;
+                buffer_len = 282;
+                free_appblock = TRUE;
+        }	
+
+	if (buffer_len > 0) {
+		dlp_WriteAppBlock (psock, handle, buffer, buffer_len);
 	}
-	
-	
-	g_message(_("Installing %s from %s..."),dbi.name, src_file);
+	if (free_appblock) {
+		free (buffer);
+	}
+		
+	g_message(_("Installing %s from %s..."), dbi.name, src_file);
 	gnome_pilot_conduit_send_message(GNOME_PILOT_CONDUIT(conduit),
 					 _("Installing %s..."),
 					 dbi.name);
 
 	
-	buffer = g_new0 (unsigned char, 0xffff);
-	remote.buffer = buffer;
-	
+	index = 0;
+	wrote = 0;
 	do {
 		unsigned long type;
 		int id;
@@ -262,57 +302,87 @@ gnome_real_pilot_conduit_file_install_db (GnomePilotConduitFile *conduit,
 		if (dbi.flags & dlpDBFlagResource) {
 			keep_reading = (pi_file_read_resource (file,
 							       index, 
-							       remote.buffer,
-							       &remote.len,
+							       &buffer,
+							       &buffer_len,
 							       &type,
 							       &id) >= 0);
+#ifdef DEBUG_REC_IO
+			g_message ("read resource id = %d, type = %s, size %d, index %d/%d",
+				   id, pi_unmktag (type), buffer_len,
+				   index+1, entries);
+#endif
+
+			if (buffer_len == 0) continue;
+
+			if (buffer_len > 65535) {
+				result = -1;
+			} else {				
+				if (type == pi_mktag('b', 'o', 'o', 't')) {
+					reset = 1;
+				}
+								
+				if (keep_reading) {				
+					err = dlp_WriteResource (psock,
+								 handle,
+								 type,
+								 id,
+								 buffer,
+								 buffer_len);					
+					
+					if (err < 0) {
+						g_warning ("error in writing to db");
+						result = -1;
+					} else {
+						wrote++;
+#ifdef DEBUG_REC_IO
+						g_message ("write resource id = %d, type = %s, size %d, index %d/%d",
+							   id, pi_unmktag (type), buffer_len,
+							   index+1, entries);
+#endif
+					}
 
-			if (type == pi_mktag('b', 'o', 'o', 't')) {
-				reset = 1;
-			}
-			
-			g_message ("appending resource %d (size %d), index %d/%d",
-				   id, remote.len,
-				   index, entries);
-			
-			if (keep_reading) {				
-				err = dlp_WriteResource (psock,
-							 handle,
-							 type,
-							 id,
-							 remote.buffer,
-							 remote.len);
-				
-				if (err < 0) {
-					g_warning ("error in writing to db");
-					result = -1;
 				}
 			}
 		} else {
 			keep_reading = (pi_file_read_record (file,
 							     index,
-							     remote.buffer,
+							     &remote.buffer,
 							     &remote.len,
 							     &remote.flags,
 							     &remote.catID,
 							     &remote.recID) >= 0);
 			
-			g_message ("appending record %d (size %d), index %d/%d",
+#ifdef DEBUG_REC_IO
+			g_message ("read record id = %d (size %d), index %d/%d",
 				   remote.recID, remote.len,
-				   index, entries);
+				   index+1, entries);
+#endif
 			
-			if (keep_reading) {
-				err = dlp_WriteRecord (psock,
-						       handle,
-						       remote.flags,
-						       remote.recID,
-						       remote.catID,
-						       remote.buffer,
-						       remote.len,
-						       0);
-				if (err < 0) {
-					g_warning ("error in writing to db");
-					result = -1;
+			if (remote.len == 0) continue;
+
+			if (remote.len > 65535) {
+				result = -1;
+			} else {
+				if (keep_reading) {
+					err = dlp_WriteRecord (psock,
+							       handle,
+							       remote.flags,
+							       remote.recID,
+							       remote.catID,
+							       remote.buffer,
+							       remote.len,
+							       0);
+					if (err < 0) {
+						g_warning ("error in writing to db");
+						result = -1;
+					} else {
+						wrote++;
+#ifdef DEBUG_REC_IO
+						g_message ("write record id = %d (size %d), index %d/%d",
+							   remote.recID, remote.len,
+							   index+1, entries);
+#endif
+					}
 				}
 			}
 			
@@ -325,22 +395,30 @@ gnome_real_pilot_conduit_file_install_db (GnomePilotConduitFile *conduit,
 
 	dlp_CloseDB (psock,handle);
 
-	if (result == 0) {
-		if (rm) {
-			unlink (src_file);
+	{
+		char *log;
+		g_message ("Wrote %d of %d %s, which is %s",
+			   wrote, entries, dbi.flags & dlpDBFlagResource ? "resources" : "records",
+			   wrote == entries ? "good" : "BAD");
+			   
+		if (result == 0) {			
+			log = g_strdup_printf ("Installed %s\n ", dbi.name);
+			if (rm) {
+				unlink (src_file);
+			}
+			
+			if (reset) {
+				g_message ("dlp_ResetSystem (%d)", psock);
+				dlp_ResetSystem (psock);
+			}
+		} else {
+			log = g_strdup_printf ("Install of %s failed\n ", dbi.name);
+			dlp_DeleteDB (psock, cardno, dbi.name);
 		}
-		
-		if (reset) {
-			g_message ("dlp_ResetSystem (%d)", psock);
-			dlp_ResetSystem (psock);
-		}
-	} else {
-		dlp_DeleteDB (psock, cardno, dbi.name);
+		dlp_AddSyncLogEntry (psock, log);
+		g_free (log);
 	}
 
- exit_and_free:	
-	g_free (buffer);
-
  close_and_exit:
 	pi_file_close (file);
 
@@ -356,9 +434,8 @@ conduit_load_gpilot_conduit (GPilotPilot *pilot)
 	retval = GNOME_PILOT_CONDUIT (gnome_pilot_conduit_file_new (pilot));
 	g_assert (retval != NULL);
 
-	gtk_signal_connect (GTK_OBJECT (retval), 
-			    GNOME_PILOT_CONDUIT_FILE_SIGNAL_INSTALL_DB (gnome_real_pilot_conduit_file_install_db), 
-			    NULL);       
+	gnome_pilot_conduit_file_connect__install_db (GNOME_PILOT_CONDUIT_FILE(retval),
+	    gnome_real_pilot_conduit_file_install_db, NULL);
 
 	return retval;
 }
