@@ -21,6 +21,10 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <glib.h>
 #include <gmodule.h>
 #include <pi-socket.h>
@@ -294,22 +298,37 @@ iterate_dbs (gint pfd,
 	GPilotPilot *pilot_info;
 	int error = 0;
 	int index = 0;
-       
+	int result;
+
 	pilot_info = gpilot_find_pilot_by_id (pu->userID, context->pilots);
 
 	orbed_notify_daemon_message (pilot_info->name, NULL, _("Collecting synchronization info..."));
 
 	while (1) {
 		GnomePilotDBInfo *dbinfo;
-		
+#ifdef PILOT_LINK_0_12
+		pi_buffer_t *pi_buf;
+#endif
 		dbinfo = g_new0 (GnomePilotDBInfo, 1);
+
+#ifdef PILOT_LINK_0_12
+		pi_buf = pi_buffer_new (sizeof (struct DBInfo));
+		result = dlp_ReadDBList (pfd, 0, dlpDBListRAM, index, pi_buf);
+#else
+		result = dlp_ReadDBList (pfd, 0, dlpDBListRAM, index, PI_DBINFO (dbinfo));
+#endif
 		/* load next dbinfo block */
-		if (dlp_ReadDBList (pfd, 0, dlpDBListRAM, index, PI_DBINFO (dbinfo)) < 0) {
+		if (result < 0) {
 			/* is <0, there are no more databases, break
                            out so we can save the list */
+			g_free (dbinfo);
 			break;
 		}
 
+#ifdef PILOT_LINK_0_12
+		memcpy (dbinfo, pi_buf->data, sizeof (struct DBInfo));
+		pi_buffer_free (pi_buf);
+#endif
 		index = PI_DBINFO (dbinfo)->index + 1;
 
 		dbs = g_list_append (dbs, dbinfo);
@@ -424,7 +443,7 @@ conduit_copy_to_pilot (GnomePilotConduitStandard *conduit, GnomePilotDBInfo *dbi
 	g_return_val_if_fail (conduit != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PILOT_CONDUIT_STANDARD (conduit), -1);
 
-	write_sync_type_line_to_pilot (dbinfo, GNOME_PILOT_CONDUIT (conduit), _("Copy to pilot %s\n"));
+	write_sync_type_line_to_pilot (dbinfo, GNOME_PILOT_CONDUIT (conduit), _("Copy to PDA %s\n"));
 
 	orbed_notify_conduit_start (dbinfo->pilotInfo->name,
 				    GNOME_PILOT_CONDUIT (conduit),
@@ -442,7 +461,7 @@ conduit_copy_from_pilot (GnomePilotConduitStandard *conduit, GnomePilotDBInfo *d
 	g_return_val_if_fail (conduit != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PILOT_CONDUIT_STANDARD (conduit), -1);
 
-	write_sync_type_line_to_pilot (dbinfo, GNOME_PILOT_CONDUIT (conduit), _("Copy from pilot %s\n"));
+	write_sync_type_line_to_pilot (dbinfo, GNOME_PILOT_CONDUIT (conduit), _("Copy from PDA %s\n"));
 
 	orbed_notify_conduit_start (dbinfo->pilotInfo->name,
 				   GNOME_PILOT_CONDUIT (conduit),
@@ -460,7 +479,7 @@ conduit_merge_to_pilot (GnomePilotConduitStandard *conduit, GnomePilotDBInfo *db
 	g_return_val_if_fail (conduit != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PILOT_CONDUIT_STANDARD (conduit), -1);
 
-	write_sync_type_line_to_pilot (dbinfo, GNOME_PILOT_CONDUIT (conduit), _("Merge to pilot %s\n"));
+	write_sync_type_line_to_pilot (dbinfo, GNOME_PILOT_CONDUIT (conduit), _("Merge to PDA %s\n"));
 
 	orbed_notify_conduit_start (dbinfo->pilotInfo->name,
 				   GNOME_PILOT_CONDUIT (conduit),
@@ -478,7 +497,7 @@ conduit_merge_from_pilot (GnomePilotConduitStandard *conduit, GnomePilotDBInfo *
 	g_return_val_if_fail (conduit != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PILOT_CONDUIT_STANDARD (conduit), -1);
 
-	write_sync_type_line_to_pilot (dbinfo, GNOME_PILOT_CONDUIT (conduit), _("Merge from pilot %s\n"));
+	write_sync_type_line_to_pilot (dbinfo, GNOME_PILOT_CONDUIT (conduit), _("Merge from PDA %s\n"));
 
 	orbed_notify_conduit_start (dbinfo->pilotInfo->name,
 				   GNOME_PILOT_CONDUIT (conduit),
@@ -761,7 +780,7 @@ gpilot_start_unknown_restore (int pfd,
 	if (result) {
 		struct PilotUser pu;
 		char *tmp;
-		tmp = g_strdup_printf (_("Setting user/id to %d/%s..."), pilot->pilot_id, pilot->pilot_username);
+		tmp = g_strdup_printf (_("Setting id/owner to %d/%s..."), pilot->pilot_id, pilot->pilot_username);
 		orbed_notify_daemon_message (pilot->name, NULL, tmp);
 		g_snprintf (pu.username, 127, "%s", pilot->pilot_username);
 		pu.userID = pilot->pilot_id;
@@ -862,11 +881,12 @@ gboolean gpilot_initial_synchronize_operations (int pilot_socket,
 	g_message ("Pilot has %d entries in conduit queue", g_list_length (request_list));
 	if (g_list_length (request_list)) {
 		carrier.conduit_list = g_list_copy (conduit_list);
-		carrier.conduit_list = g_list_concat (carrier.conduit_list, backup_conduit_list);
-		carrier.conduit_list = g_list_concat (carrier.conduit_list, file_conduit_list);
+		carrier.conduit_list = g_list_concat (carrier.conduit_list, g_list_copy (backup_conduit_list));
+		carrier.conduit_list = g_list_concat (carrier.conduit_list, g_list_copy (file_conduit_list));
 		do_normal_sync_stuff = FALSE;
 		g_list_foreach (request_list,(GFunc)run_conduit_sync_foreach,&carrier);
 		g_list_free (carrier.conduit_list);
+
 	}
 
 	return do_normal_sync_stuff;
